@@ -40,11 +40,12 @@ ModuleNexus<Globals> KeychainCore::globals;
 #pragma mark  Constructor/Destructor 
 
 Globals::Globals() :
-mUI(true), mACLFactory(CssmAllocator::standard())
+apiLock(Mutex::recursive),
+mUI(true)
 {
 }
 
 const AccessCredentials * Globals::credentials() 
 {
-	return (mUI ? mACLFactory.keychainPromptCredentials() : mACLFactory.nullCredentials()); 
+	return (mUI ? mACLFactory.promptCred() : mACLFactory.nullCred()); 
 }
