@@ -31,6 +31,11 @@ using CodeSigning::OSXCode;
 namespace Security {
 namespace SecurityServer {
 
+//
+// Static callback stuff
+//
+ClientSession::DidChangeKeyAclCallback *ClientSession::mCallback = NULL;
+void *ClientSession::mCallbackContext = NULL;
 
 //
 // The process-global object
@@ -38,7 +43,7 @@ namespace SecurityServer {
 UnixPlusPlus::StaticForkMonitor ClientSession::mHasForked;
 ModuleNexus<ClientSession::Global> ClientSession::mGlobal;
 bool ClientSession::mSetupSession;
-
+const char *ClientSession::mContactName;
 
 //
 // Construct a client session
@@ -55,6 +60,13 @@ ClientSession::~ClientSession()
 { }
 
 
+void
+ClientSession::registerForAclEdits(DidChangeKeyAclCallback *callback, void *context)
+{
+	mCallback = callback;
+	mCallbackContext = context;
+}
+
 //
 // Activate a client session: This connects to the SecurityServer and executes
 // application authentication
@@ -65,7 +77,7 @@ void ClientSession::activate()
 	// (that has not exec'ed), our apparent connection to SecurityServer
 	// is just a mirage, and we better reset it.
 	if (mHasForked()) {
-		debug("SSclnt", "process has forked (now pid=%d) - resetting connection object", getpid());
+		secdebug("SSclnt", "process has forked (now pid=%d) - resetting connection object", getpid());
 		mGlobal.reset();
 	}
 		
@@ -74,14 +86,32 @@ void ClientSession::activate()
     Thread &thread = global.thread();
     if (!thread) {
 		// first time for this thread - use abbreviated registration
-		IPCN(ucsp_client_setup(UCSP_ARGS, mach_task_self(), ""));
+		IPCN(ucsp_client_setupThread(UCSP_ARGS, mach_task_self()));
         thread.registered = true;
         global.serverPort.requestNotify(thread.replyPort, MACH_NOTIFY_DEAD_NAME, true);
-        debug("SSclnt", "Thread registered with SecurityServer");
+        secdebug("SSclnt", "Thread registered with %s", mContactName);
 	}
 }
 
 
+//
+// The contactName method allows the caller to explicitly override the bootstrap
+// name under which SecurityServer is located. Use this only with great caution,
+// and probably only for debugging.
+// Note that no explicit locking is done here. It is the caller's responsibility
+// to make sure this is called from thread-safe context before the real dance begins.
+//
+void ClientSession::contactName(const char *name)
+{
+	mContactName = name;
+}
+
+const char *ClientSession::contactName() const
+{
+	return mContactName;
+}
+
+
 //
 // Construct the process-global state object.
 // The ModuleNexus construction magic will ensure that this happens uniquely
@@ -92,36 +122,42 @@ void ClientSession::activate()
 ClientSession::Global::Global()
 {
     // find server port
-	Bootstrap myBootstrap;
-    serverPort = myBootstrap.lookup("SecurityServer");
-	debug("SSclnt", "contacting SecurityServer at port %d", serverPort.port());
+	IFDEBUG(if (!mContactName) mContactName = getenv(SECURITYSERVER_BOOTSTRAP_ENV));
+	if (!mContactName)
+		mContactName = SECURITYSERVER_BOOTSTRAP_NAME;
+    secdebug("SSclnt", "Locating %s", mContactName);
+    serverPort = Bootstrap().lookup(mContactName);
+	secdebug("SSclnt", "contacting %s at port %d", mContactName, serverPort.port());
     
     // send identification/setup message
     string extForm;
     try {
         myself = OSXCode::main();
         extForm = myself->encode();
-        debug("SSclnt", "my OSXCode extForm=%s", extForm.c_str());
+        secdebug("SSclnt", "my OSXCode extForm=%s", extForm.c_str());
     } catch (...) {
         // leave extForm empty
-        debug("SSclnt", "failed to obtain my own OSXCode");
+        secdebug("SSclnt", "failed to obtain my own OSXCode");
     }
+
+	ClientSetupInfo info = { SSPROTOVERSION };
+	
     // cannot use UCSP_ARGS here because it uses mGlobal() -> deadlock
     Thread &thread = this->thread();
 	
 	if (mSetupSession) {
-		debug("SSclnt", "sending session setup request");
+		secdebug("SSclnt", "sending session setup request");
 		mSetupSession = false;
 		IPCN(ucsp_client_setupNew(serverPort, thread.replyPort, &rcode,
-			mach_task_self(), extForm.c_str(), &serverPort.port()));
-		debug("SSclnt", "new session server port is %d", serverPort.port());
-	} else {	
+			mach_task_self(), info, extForm.c_str(), &serverPort.port()));
+		secdebug("SSclnt", "new session server port is %d", serverPort.port());
+	} else {
 		IPCN(ucsp_client_setup(serverPort, thread.replyPort, &rcode,
-			mach_task_self(), extForm.c_str()));
+			mach_task_self(), info, extForm.c_str()));
 	}
     thread.registered = true;	// as a side-effect of setup call above
 	serverPort.requestNotify(thread.replyPort, MACH_NOTIFY_DEAD_NAME, true);
-	debug("SSclnt", "contact with SecurityServer established");
+	secdebug("SSclnt", "contact with %s established", mContactName);
 }
 
 
@@ -131,10 +167,22 @@ ClientSession::Global::Global()
 void ClientSession::terminate()
 {
 	// currently defunct
-	debug("SSclnt", "ClientSession::terminate() call ignored");
+	secdebug("SSclnt", "ClientSession::terminate() call ignored");
+}
+
+
+void ClientSession::addApplicationAclSubject(KeyHandle key, CSSM_ACL_AUTHORIZATION_TAG tag)
+{
+	/* Notify our client if they are interested. */
+	if (mCallback && mCallbackContext)
+	{
+		secdebug("keyacl", "ClientSession::addApplicationAclSubject(keyHandle: %lu tag: %lu)", key, tag);
+		mCallback(mCallbackContext, *this, key, tag);
+	}
+	else
+		secdebug("keyacl", "ClientSession::addApplicationAclSubject() with NULL mCallback");
 }
 
 
 } // end namespace SecurityServer
-
 } // end namespace Security
